@@ -3,7 +3,7 @@
     settings of Smb4K
                              -------------------
     begin                : Sa Okt 30 2004
-    copyright            : (C) 2004-2007 by Alexander Reinholdt
+    copyright            : (C) 2004-2008 by Alexander Reinholdt
     email                : dustpuppy@users.berlios.de
  ***************************************************************************/
 
@@ -25,51 +25,65 @@
  ***************************************************************************/
 
 // Qt includes
-#include <qgroupbox.h>
-#include <qbuttongroup.h>
-#include <qlabel.h>
-#include <qlayout.h>
-#include <qradiobutton.h>
-#include <qcheckbox.h>
+#include <QGridLayout>
+#include <QGroupBox>
+#include <QLabel>
+#include <QCheckBox>
 
 // KDE includes
 #include <klocale.h>
 #include <kpushbutton.h>
 #include <kguiitem.h>
+#include <kcombobox.h>
 
 // application specific includes
 #include "smb4ksuperuseroptions.h"
-#include "../core/smb4ksettings.h"
+#include <core/smb4ksettings.h>
 
 
-Smb4KSuperUserOptions::Smb4KSuperUserOptions( QWidget *parent, const char *name )
-: QWidget( parent, name )
+Smb4KSuperUserOptions::Smb4KSuperUserOptions( QWidget *parent )
+: QWidget( parent )
 {
-  QGridLayout *grid =           new QGridLayout( this );
-  grid->setSpacing( 10 );
+  QGridLayout *layout           = new QGridLayout( this );
+  layout->setSpacing( 5 );
+  layout->setMargin( 0 );
+
+  // Actions
+  QGroupBox *actions_box        = new QGroupBox( i18n( "Actions" ), this );
+
+  QGridLayout *actions_layout   = new QGridLayout( actions_box );
+  actions_layout->setSpacing( 5 );
 
-  QButtonGroup *suid_actions =  new QButtonGroup( 1, QButtonGroup::Horizontal, i18n( "Actions" ), this, "SUIDActions" );
-  suid_actions->setInsideSpacing( 5 );
 #ifdef __linux__
-  (void) new QCheckBox( i18n( "Use super user privileges to force the unmounting of (inaccessible) shares" ), suid_actions, "kcfg_UseForceUnmount" );
+  QCheckBox *force_unmounting   = new QCheckBox( Smb4KSettings::self()->useForceUnmountItem()->label(),
+                                  actions_box );
+  force_unmounting->setObjectName( "kcfg_UseForceUnmount" );
 #endif
-  (void) new QCheckBox( i18n( "Use super user privileges to mount and unmount shares" ), suid_actions, "kcfg_AlwaysUseSuperUser" );
+  QCheckBox *use_privileges     = new QCheckBox( Smb4KSettings::self()->alwaysUseSuperUserItem()->label(),
+                                  actions_box );
+  use_privileges->setObjectName( "kcfg_AlwaysUseSuperUser" );
 
-  QSpacerItem *spacer1 =        new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Preferred );
-  KGuiItem remove_item =        KGuiItem(  i18n( "Remove Entries" ), "editdelete",
-                                i18n( "Remove entries from the configuration file" ),
-                                i18n( "All entries that were written by Smb4K will be removed from the sudoers file. Additionally, all your choices under \"Actions\" will be cleared." ) );
-  KPushButton *remove =         new KPushButton( remove_item, this, "RemoveButton" );
+#ifdef __linux__
+  actions_layout->addWidget( force_unmounting, 0, 0, 0 );
+  actions_layout->addWidget( use_privileges, 1, 0, 0 );
+#else
+  actions_layout->addWidget( use_privileges, 0, 0, 0 );
+#endif
 
-  QSpacerItem *spacer2 =        new QSpacerItem( 0, 0, QSizePolicy::Preferred, QSizePolicy::Expanding );
+  KGuiItem remove_item          = KGuiItem(  i18n( "Remove Entries" ), "edit-clear",
+                                  i18n( "Remove entries from the configuration file" ),
+                                  i18n( "Depending on your choice under \"Programs\", all entries that were written by Smb4K will be removed either from /etc/super.tab or /etc/sudoers. Additionally, all your choices under \"Actions\" will be cleared." ) );
 
-  grid->addMultiCellWidget( suid_actions, 0, 0, 0, 3, 0 );
-  grid->addMultiCell( spacer1, 1, 1, 0, 2, 0 );
-  grid->addWidget( remove, 1, 3, 0 );
-  grid->addMultiCell( spacer2, 2, 2, 0, 3, 0 );
+  KPushButton *remove           = new KPushButton( remove_item, this );
 
-  connect( remove, SIGNAL( clicked() ),
-           this,   SLOT( slotRemoveClicked() ) );
+  QSpacerItem *spacer = new QSpacerItem( 10, 10, QSizePolicy::Preferred, QSizePolicy::Expanding );
+
+  layout->addWidget( actions_box, 0, 0, 1, 6, 0 );
+  layout->addWidget( remove, 1, 5, 0 );
+  layout->addItem( spacer, 2, 0, 1, 6, 0 );
+
+  connect( remove, SIGNAL( clicked( bool ) ),
+           this,   SLOT( slotRemoveClicked( bool ) ) );
 }
 
 
@@ -82,7 +96,7 @@ Smb4KSuperUserOptions::~Smb4KSuperUserOptions()
 // SLOT IMPLEMENTATIONS
 /////////////////////////////////////////////////////////////////////////////
 
-void Smb4KSuperUserOptions::slotRemoveClicked()
+void Smb4KSuperUserOptions::slotRemoveClicked( bool /*checked*/ )
 {
   // Emit the signal that the removal of the super user
   // entries has been requested.
