@@ -25,12 +25,12 @@
  ***************************************************************************/
 
 // Qt includes
-#include <qdir.h>
-#include <qfile.h>
-#include <qtextcodec.h>
-#include <qstring.h>
-#include <qcstring.h>
-#include <qvaluelist.h>
+#include <QDir>
+#include <QFile>
+#include <QTextCodec>
+#include <QString>
+#include <QByteArray>
+#include <QStringList>
 
 // KDE includes
 #include <kaboutdata.h>
@@ -65,17 +65,17 @@ int createLockFile()
   // Determine the directory where to write the lock file. First, try
   // /var/lock and than /var/tmp. If that does not work either, fall
   // back to /tmp.
-  QValueList<QCString> dirs;
+  QList<QByteArray> dirs;
   dirs << "/var/lock";
   dirs << "/var/tmp";
   dirs << "/tmp";
 
   struct stat buf;
 
-  for ( QValueList<QCString>::ConstIterator d = dirs.begin(); d != dirs.end(); ++d )
+  for ( int i = 0; i < dirs.size(); ++i )
   {
     // First check if the directory is available and writable
-    if ( lstat( *d, &buf ) == -1 )
+    if ( lstat( dirs.at( i ), &buf ) == -1 )
     {
       int error_number = errno;
 
@@ -92,15 +92,15 @@ int createLockFile()
     // Get the ids of the groups the user is in and check if
     // on of them matches buf.st_gid.
     KUser user( geteuid() );
-    QValueList<KUserGroup> gids = user.groups();
+    QList<KUserGroup> gids = user.groups();
     gid_t sup_gid = 65534; // set this to gid 'nobody' for initialization
     bool found_gid = false;
 
-    for ( QValueList<KUserGroup>::ConstIterator g = gids.begin(); g != gids.end(); ++g )
+    for ( int j = 0; j < gids.size(); ++j )
     {
-      if ( (*g).gid() == buf.st_gid )
+      if ( gids.at( j ).gid() == buf.st_gid )
       {
-        sup_gid = (*g).gid();
+        sup_gid = gids.at( j ).gid();
         found_gid = false;
 
         break;
@@ -118,7 +118,7 @@ int createLockFile()
         (found_gid && buf.st_gid == sup_gid && (buf.st_mode & 00060) == (S_IWGRP | S_IRGRP)) /* group */ ||
         ((buf.st_mode & 00006) == (S_IWOTH | S_IROTH)) /* others */) )
     {
-      lock_file.setName( (*d)+"/smb4k.lock" );
+      lock_file.setFileName( dirs.at( i )+"/smb4k.lock" );
 
       break;
     }
@@ -130,14 +130,25 @@ int createLockFile()
 
   if ( !lock_file.exists() )
   {
-    if ( lock_file.open( IO_WriteOnly ) )
+    if ( lock_file.open( QIODevice::WriteOnly | QIODevice::Text ) )
     {
       QTextStream ts( &lock_file );
       // Note: With Qt 4.3 this seems to be obsolete, we'll keep
       // it for now.
       ts.setCodec( QTextCodec::codecForLocale() );
 
+#if QT_VERSION >= 0x040400
+      if ( kapp )
+      {
+        ts << kapp->applicationPid() << endl;
+      }
+      else
+      {
+        ts << "0" << endl;
+      }
+#else
       ts << "0" << endl;
+#endif
 
       lock_file.close();
     }
@@ -175,9 +186,9 @@ const QByteArray findFile( const QString &filename )
 
   QString canonical_path;
 
-  for ( QStringList::Iterator p = paths.begin(); p != paths.end(); ++p )
+  for ( int i = 0; i < paths.size(); ++i )
   {
-    QDir::setCurrent( *p );
+    QDir::setCurrent( paths.at( i ) );
 
     if ( QFile::exists( filename ) )
     {
@@ -191,14 +202,14 @@ const QByteArray findFile( const QString &filename )
     }
   }
 
-  return canonical_path.local8Bit();
+  return canonical_path.toLocal8Bit();
 }
 
-bool checkUsers( const QCStringList &list )
+bool checkUsers( const QStringList &list )
 {
-  for ( QCStringList::ConstIterator it = list.begin(); it != list.end(); ++it )
+  for ( int i = 0; i < list.size(); ++i )
   {
-    if ( getpwnam( *it ) == NULL )
+    if ( getpwnam( list.at( i ).toLocal8Bit() ) == NULL )
     {
       return false;
     }
@@ -230,15 +241,15 @@ int checkFile( const QByteArray &path )
   // Get the ids of the groups the user is in and check if
   // on of them matches buf.st_gid.
   KUser user( geteuid() );
-  QValueList<KUserGroup> gids = user.groups();
+  QList<KUserGroup> gids = user.groups();
   gid_t sup_gid = 65534; // set this to gid 'nobody' for initialization
   bool found_gid = false;
 
-  for ( QValueList<KUserGroup>::ConstIterator g = gids.begin(); g != gids.end(); ++g )
+  for ( int i = 0; i < gids.size(); ++i )
   {
-    if ( (*g).gid() == buf.st_gid )
+    if ( gids.at( i ).gid() == buf.st_gid )
     {
-      sup_gid = (*g).gid();
+      sup_gid = gids.at( i ).gid();
       found_gid = false;
 
       break;
@@ -291,26 +302,27 @@ bool findUtilityPrograms()
 int main ( int argc, char *argv[] )
 {
   KAboutData aboutData( "smb4k_sudowriter",
-                        I18N_NOOP( "smb4k_sudowriter" ),
-                        "0.1",
-                        I18N_NOOP( description ),
+                        "smb4k",
+                        ki18n( "smb4k_sudowriter" ),
+                        "0.2",
+                        ki18n( description ),
                         KAboutData::License_GPL_V2,
-                        I18N_NOOP( authors ),
-                        0,
+                        ki18n( authors ),
+                        KLocalizedString(),
                         "http://smb4k.berlios.de",
                         "smb4k-bugs@lists.berlios.de" );
 
   KCmdLineArgs::init( argc, argv, &aboutData );
 
-  static const KCmdLineOptions options[] =
-  {
-    { "adduser <user>",     I18N_NOOP( "Adds an user to the sudoers file" ), 0 },
-    { "removeuser <user>",  I18N_NOOP( "Removes an user from the sudoers file" ), 0 },
-    KCmdLineLastOption
-  };
+  KCmdLineOptions options;
+  options.add( "adduser <user>",
+               ki18n( "Adds an user to the sudoers file" ),
+               0 );
+  options.add( "removeuser <user>",
+               ki18n( "Removes an user from the sudoers file" ),
+               0 );
 
   KCmdLineArgs::addCmdLineOptions( options );
-  KApplication::addCmdLineOptions();
 
   KApplication app( false /* no GUI */ );
 
@@ -349,13 +361,13 @@ int main ( int argc, char *argv[] )
   KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
 
   // Check that everything is OK.
-  QCStringList adduser    = args->getOptionList( "adduser" );
-  QCStringList removeuser = args->getOptionList( "removeuser" );
+  QStringList adduser    = args->getOptionList( "adduser" );
+  QStringList removeuser = args->getOptionList( "removeuser" );
 
   // Throw an error if no argument was provided.
   if ( adduser.isEmpty() && removeuser.isEmpty() )
   {
-    KCmdLineArgs::usage( i18n( "No arguments given." ) );
+    KCmdLineArgs::usageError( i18n( "No arguments given." ) );
     removeLockFile();
     exit( EXIT_FAILURE );
   }
@@ -436,38 +448,19 @@ int main ( int argc, char *argv[] )
     // Do nothing
   }
 
+  // Now work with the sudoers file.
+  QFile file( path );
+
   // Save the original permissions for later.
-  struct stat file_stat;
-
-  if ( stat( path, &file_stat ) == -1 )
-  {
-    int error_number = errno;
-    cerr << argv[0] << ": " << I18N_NOOP( "Could not stat the file " ) << path << "." << endl;
-    cerr << argv[0] << ": " << strerror( error_number ) << endl;
-    cerr << argv[0] << ": " << I18N_NOOP( "Aborting." ) << endl;
-    removeLockFile();
-    exit( EXIT_FAILURE );
-  }
-
-  mode_t perms = file_stat.st_mode;
+  QFile::Permissions perms = file.permissions();
 
   // Temporarily give the *owner* the permission to
-  // read and write to the file.
-  if ( chmod( path, S_IRUSR | S_IWUSR ) == -1 )
-  {
-    int error_number = errno;
-    cerr << argv[0] << ": " << I18N_NOOP( "Could not change file permissions for " ) << path << endl;
-    cerr << argv[0] << ": " << strerror( error_number ) << endl;
-    cerr << argv[0] << ": " << I18N_NOOP( "Aborting." ) << endl;
-    removeLockFile();
-    exit( EXIT_FAILURE );
-  }
-  
-  // Read the contents of the file.
-  QFile file( path );
+  // write to the file.
+  file.setPermissions( QFile::WriteOwner | QFile::ReadOwner );
+
   QStringList contents;
 
-  if ( file.open( IO_ReadOnly ) )
+  if ( file.open( QIODevice::ReadOnly | QIODevice::Text ) )
   {
     QTextStream ts( &file );
     // Note: With Qt 4.3 this seems to be obsolete, we'll keep
@@ -476,50 +469,31 @@ int main ( int argc, char *argv[] )
 
     while ( !ts.atEnd() )
     {
-      contents.append( ts.readLine() );
+      contents.append( ts.readLine( 0 ) );
     }
 
     file.close();
-    
-    if ( chmod( path, perms ) == -1 )
-    {
-      int error_number = errno;
-      cerr << argv[0] << ": " << I18N_NOOP( "Could not change file permissions for " ) << path << endl;
-      cerr << argv[0] << ": " << strerror( error_number ) << endl;
-      cerr << argv[0] << ": " << I18N_NOOP( "Aborting." ) << endl;
-      removeLockFile();
-      exit( EXIT_FAILURE );
-    }
+    file.setPermissions( perms );
   }
   else
   {
-    if ( chmod( path, perms ) == -1 )
-    {
-      int error_number = errno;
-      cerr << argv[0] << ": " << I18N_NOOP( "Could not change file permissions for " ) << path << endl;
-      cerr << argv[0] << ": " << strerror( error_number ) << endl;
-      cerr << argv[0] << ": " << I18N_NOOP( "Aborting." ) << endl;
-      removeLockFile();
-      exit( EXIT_FAILURE );
-    }
-       
-    cerr << argv[0] << ": " << file.errorString().local8Bit().data() << endl;
+    file.setPermissions( perms );
+    cerr << argv[0] << ": " << file.errorString().toLocal8Bit().data() << endl;
     removeLockFile();
     exit( EXIT_FAILURE );
   }
 
   // Find the beginning and the end of the entries in
   // the sudoers file:
-  QStringList::Iterator begin = contents.find( "# Entries for Smb4K users." );
-  QStringList::Iterator end = contents.find( "# End of Smb4K user entries." );
+  int begin = contents.indexOf( "# Entries for Smb4K users.", 0 );
+  int end = contents.lastIndexOf( "# End of Smb4K user entries.", -1 );
 
   bool write = false;
 
   // Add user(s).
   if ( !adduser.isEmpty() )
   {
-    if ( contents.grep( "# Entries for Smb4K users." ).count() == 0 &&
-         contents.grep( "# End of Smb4K user entries." ).count() == 0 )
+    if ( begin == -1 && end == -1 )
     {
       // Get the hostname.
       size_t hostnamelen = 255;
@@ -541,7 +515,7 @@ int main ( int argc, char *argv[] )
       delete [] hn;
 
       // Add the new entries.
-      if ( !contents.last().stripWhiteSpace().isEmpty() )
+      if ( !contents.last().trimmed().isEmpty() )
       {
         contents.append( "" );
       }
@@ -550,19 +524,9 @@ int main ( int argc, char *argv[] )
         // Do not add empty line to the end.
       }
 
-      QString users;
-      
-      for ( QCStringList::Iterator it = adduser.begin(); it != adduser.end(); ++it )
-      {
-        users.append( QString::fromLocal8Bit( *it, -1 ) );
-        users.append( "," );
-      }
-      
-      users.truncate( users.length() - 1 );
-
       contents.append( "# Entries for Smb4K users." );
       contents.append( "# Generated by Smb4K. Please do not modify!" );
-      contents.append( "User_Alias\tSMB4KUSERS = "+users );
+      contents.append( "User_Alias\tSMB4KUSERS = "+QString( "%1" ).arg( adduser.join( "," ) ) );
       contents.append( "Defaults:SMB4KUSERS\tenv_keep += \"PASSWD USER\"" );
       contents.append( "SMB4KUSERS\t"+hostname+" = NOPASSWD: "
                        +KGlobal::dirs()->findResource( "exe", "smb4k_kill" ) );
@@ -574,18 +538,17 @@ int main ( int argc, char *argv[] )
 
       write = true;
     }
-    else if ( contents.grep( "# Entries for Smb4K users." ).count() == 1 &&
-              contents.grep( "# End of Smb4K user entries." ).count() == 1 )    
+    else if ( begin != -1 && end != -1 )
     {
-      for ( QStringList::Iterator it = begin; it != end; ++it )
+      for ( int i = begin; i != end; ++i )
       {
-        if ( (*it).startsWith( "User_Alias\tSMB4KUSERS" ) )
+        if ( contents.at( i ).startsWith( "User_Alias\tSMB4KUSERS" ) )
         {
-          for ( QCStringList::Iterator i = adduser.begin(); i != adduser.end(); ++i )
+          for ( int j = 0; j < adduser.size(); ++j )
           {
-            if ( !(*it).contains( QString::fromLocal8Bit( *i, -1 ) ) )
+            if ( !contents.at( i ).contains( adduser.at( j ) ) )
             {
-              (*it).append( ","+QString::fromLocal8Bit( *i, -1 ) );
+              contents[i].append( ","+adduser.at( j ) );
               continue;
             }
             else
@@ -619,25 +582,31 @@ int main ( int argc, char *argv[] )
   // Remove user(s).
   if ( !removeuser.isEmpty() )
   {
-    if ( contents.grep( "# Entries for Smb4K users." ).count() != 0 &&
-         contents.grep( "# End of Smb4K user entries." ).count() != 0 )  
+    if ( begin != -1 && end != -1 )
     {
-      for ( QStringList::Iterator it = begin; it != end; ++it )
+      for ( int i = begin; i != end; ++i )
       {
-        if ( (*it).startsWith( "User_Alias\tSMB4KUSERS" ) )
+        if ( contents.at( i ).startsWith( "User_Alias\tSMB4KUSERS" ) )
         {
-          QString users = (*it).section( "=", 1, 1 ).stripWhiteSpace();
+          QString users = contents.at( i ).section( "=", 1, 1 ).trimmed();
 
           if ( !users.contains( "," ) )
           {
             // In this case, there is only one user in the list. Check if
             // it is the user who requested the removal:
-            for ( QCStringList::Iterator i = removeuser.begin(); i != removeuser.end(); ++i )
+            for ( int j = 0; j < removeuser.size(); ++j )
             {
-              if ( QString::compare( users, *i ) == 0 )
+              if ( QString::compare( users, removeuser.at( j ) ) == 0 )
               {
-                contents.erase( begin, end );
-                contents.erase( end );
+                // They are equal. Remove all entries:
+                int k = begin;
+
+                while ( k != end + 1 ) // We want to remove line 'end' as well.
+                {
+                  contents.removeAt( begin );
+
+                  k++;
+                }
 
                 write = true;
 
@@ -656,14 +625,20 @@ int main ( int argc, char *argv[] )
           {
             // In this case there is more than one user in the list.
             // Remove the user who requested the removal:
-            QStringList list = QStringList::split( ",", users, false );
+            QStringList list = users.split( ",", QString::SkipEmptyParts );
+            int index = 0;
 
-            for ( QCStringList::Iterator i = removeuser.begin(); i != removeuser.end(); ++i )
+            for ( int j = 0; j < removeuser.size(); ++j )
             {
-              if ( list.remove( QString::fromLocal8Bit( *i, -1 ) ) != 0 )
+              index = list.indexOf( removeuser.at( j ), 0 );
+
+              if ( index != -1 )
               {
-                (*it).replace( users, list.join( "," ) );
+                list.removeAt( index );
+                contents[i].replace( users, list.join( "," ) );
+
                 write = true;
+
                 continue;
               }
               else
@@ -683,8 +658,7 @@ int main ( int argc, char *argv[] )
         }
       }
     }
-    else if ( contents.grep( "# Entries for Smb4K users." ).count() == 0 &&
-              contents.grep( "# End of Smb4K user entries." ).count() == 0 )
+    else if ( begin == -1 && end == -1 )
     {
       // Do nothing
     }
@@ -704,18 +678,10 @@ int main ( int argc, char *argv[] )
   if ( write )
   {
     // Temporarily give the *owner* the permission to
-    // read and write to the file.
-    if ( chmod( path, S_IRUSR | S_IWUSR ) == -1 )
-    {
-      int error_number = errno;
-      cerr << argv[0] << ": " << I18N_NOOP( "Could not change file permissions for " ) << path << endl;
-      cerr << argv[0] << ": " << strerror( error_number ) << endl;
-      cerr << argv[0] << ": " << I18N_NOOP( "Aborting." ) << endl;
-      removeLockFile();
-      exit( EXIT_FAILURE );
-    }
+    // write to the file.
+    file.setPermissions( QFile::WriteOwner | QFile::ReadOwner );
 
-    if ( file.open( IO_WriteOnly ) )
+    if ( file.open( QIODevice::WriteOnly | QIODevice::Text ) )
     {
       QTextStream ts( &file );
       // Note: With Qt 4.3 this seems to be obsolete, we'll keep
@@ -725,28 +691,12 @@ int main ( int argc, char *argv[] )
       ts << contents.join( "\n" ) << endl;
 
       file.close();
-    
-      if ( chmod( path, perms ) == -1 )
-      {
-        int error_number = errno;
-        cerr << argv[0] << ": " << I18N_NOOP( "Could not change file permissions for " ) << path << endl;
-        cerr << argv[0] << ": " << strerror( error_number ) << endl;
-        cerr << argv[0] << ": " << I18N_NOOP( "Aborting." ) << endl;
-        removeLockFile();
-        exit( EXIT_FAILURE );
-      }    
+      file.setPermissions( perms );
     }
     else
     {
-      if ( chmod( path, S_IRUSR | S_IWUSR ) == -1 )
-      {
-        int error_number = errno;
-        cerr << argv[0] << ": " << I18N_NOOP( "Could not change file permissions for " ) << path << endl;
-        cerr << argv[0] << ": " << strerror( error_number ) << endl;
-        cerr << argv[0] << ": " << I18N_NOOP( "Aborting." ) << endl;
-      }      
-      
-      cerr << argv[0] << ": " << file.errorString().local8Bit().data() << endl;
+      file.setPermissions( perms );
+      cerr << argv[0] << ": " << file.errorString().toLocal8Bit().data() << endl;
       removeLockFile();
       exit( EXIT_FAILURE );
     }
